@@ -4,8 +4,12 @@
 
 #include <r_util/r_strbuf.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct pj_t {
-	RStrBuf *sb;
+	RStrBuf sb;
 	bool is_first;
 	bool is_key;
 	char braces[R_PRINT_JSON_DEPTH_LIMIT];
@@ -28,13 +32,19 @@ R_API PJ *pj_o(PJ *j);
 R_API PJ *pj_a(PJ *j);
 /* keys, values */
 R_API PJ *pj_k(PJ *j, const char *k);
+R_API PJ *pj_knull(PJ *j, const char *k);
 R_API PJ *pj_kn(PJ *j, const char *k, ut64 n);
 R_API PJ *pj_kN(PJ *j, const char *k, st64 n);
 R_API PJ *pj_ks(PJ *j, const char *k, const char *v);
+R_API PJ *pj_ka(PJ *j, const char *k);
+R_API PJ *pj_ko(PJ *j, const char *k);
 R_API PJ *pj_ki(PJ *j, const char *k, int d);
 R_API PJ *pj_kd(PJ *j, const char *k, double d);
 R_API PJ *pj_kf(PJ *j, const char *k, float d);
 R_API PJ *pj_kb(PJ *j, const char *k, bool v);
+R_API PJ *pj_null(PJ *j);
+R_API PJ *pj_r(PJ *j, const unsigned char *v, size_t v_len);
+R_API PJ *pj_kr(PJ *j, const char *k, const unsigned char *v, size_t v_len);
 R_API PJ *pj_b(PJ *j, bool v);
 R_API PJ *pj_s(PJ *j, const char *k);
 R_API PJ *pj_n(PJ *j, ut64 n);
@@ -43,5 +53,10 @@ R_API PJ *pj_d(PJ *j, double d);
 R_API PJ *pj_f(PJ *j, float d);
 R_API PJ *pj_i(PJ *j, int d);
 R_API PJ *pj_j(PJ *j, const char *k);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
